{ ****************************************************************************
  Validat.pas                 Copyright  1997-99 DithoSoft Software Solutions
  Version 1.0
  ----------------------------------------------------------------------------
  Delphi versions: designed for Delphi 2, probably works alright with Delphi 1
                   and 3
  ----------------------------------------------------------------------------
  Features:        - prevents specified characters or other characters than
                   the specified one from being entered
                   - checks strings that are being inserted for those
                   characters and changes the strings accordingly
  ----------------------------------------------------------------------------
  How to use:      Insert a TValidatorEdit component into your form, then
                   enter the characters which should be affected into the Keys
                   property and specify how the characters are affected by
                   setting the UseKeys property to either veUseKeysAllowed or
                   veUseKeysForbidden
  ----------------------------------------------------------------------------
  Known Problems: None.
  ----------------------------------------------------------------------------
  }
unit Validat;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  {TUseKeys Structure: Internally used by TValidatorEdit}
  TUseKeys = (veUseKeysForbidden, veUseKeysAllowed);

  {TValidatorEdit Component}
  TValidatorEdit = class(TEdit)
  private
    FKeys : string; // Characters that have to be checked
    FUseKeys : TUseKeys;  // Determines what is done with the characters
    {Internal method}
    procedure CheckText;
    {Property method}
    procedure SetKeys(AKeys : string);
    procedure SetUseKeys(AUseKeys : TUseKeys);
  protected
    {Eventhandlers}
    procedure KeyPress(var Key : Char); override;
    procedure Change; override;
  published
    {Published Property}
    property Keys : string read FKeys write SetKeys;
    property UseKeys : TUseKeys read FUseKeys write SetUseKeys;
  end;

procedure Register;

implementation

{ ****************************************************************************
  TValidatorEdit
  ****************************************************************************}
{Internal method of TValidatorEdit}
procedure TValidatorEdit.CheckText;
var Count : Longint;
    FText : string;
begin
  Count := 1;
  {Copy the Text property to a dummy variable}
  FText := Text;

  {Check for characters...}
  while Count <= Length(FText) do begin
    {...that are either allowed or forbidden...}
    If FUseKeys = veUseKeysAllowed then begin
      If Pos(FText[Count], FKeys) = 0 then
        {...and delete either the other characters...}
        Delete(FText, Count, 1)
      else Inc(Count);
    end else
      If Pos(FText[Count], FKeys) <> 0 then
        {..or the specified characters from the dummy variable}
        Delete(FText, Count, 1)
      else Inc(Count);
  end;

  {...then copy the dummy variable back into the Text property}
  Text := FText;
end;

{Specify the characters either allowed or forbidden}
procedure TValidatorEdit.SetKeys(AKeys : string);
begin
  {Change the property variable}
  FKeys := AKeys;

  CheckText;
end;

{Specify what to do with those characters}
procedure TValidatorEdit.SetUseKeys(AUseKeys : TUseKeys);
begin
  {Change the property variable}
  FUseKeys := AUseKeys;

  CheckText;
end;

{Check for characters when the Text property changes, e.g. because a string
 is pasted from the clipboard}
procedure TValidatorEdit.Change;
begin
  CheckText;

  inherited;
end;

{Check what characters are being entered and suppress them if necessary}
procedure TValidatorEdit.KeyPress(var Key : Char);
begin
  { Suppress any other characters than those specified in the Keys property }
  If FUseKeys = veUseKeysAllowed then begin
    if Pos(Key, FKeys) = 0 then begin
      MessageBeep(0);
      Key := #0;
    end;
  { Suppress any characters that have been specified in the Keys property }
  end else begin
    if Pos(Key, FKeys) <> 0 then begin
      MessageBeep(0);
      Key := #0;
    end;
  end;

  inherited;
end;

{Register TValidatorEdit for use with Delphi}
procedure Register;
begin
  RegisterComponents('Freeware', [TValidatorEdit]);
end;

end.
